@@ -4,31 +4,30 @@ import (
 	"io"
 	"testing"
 
-	"github.com/elves/elvish/pkg/cli"
-	"github.com/elves/elvish/pkg/cli/term"
-	"github.com/elves/elvish/pkg/eval/vals"
-	"github.com/elves/elvish/pkg/tt"
-	"github.com/elves/elvish/pkg/ui"
+	"src.elv.sh/pkg/cli/term"
+	"src.elv.sh/pkg/cli/tk"
+	"src.elv.sh/pkg/eval"
+	. "src.elv.sh/pkg/eval/evaltest"
+	"src.elv.sh/pkg/tt"
+	"src.elv.sh/pkg/ui"
 )
 
 func TestBindingTable(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
 	evals(f.Evaler, `called = $false`)
 	evals(f.Evaler, `m = (edit:binding-table [&a={ called = $true }])`)
-	_, ok := getGlobal(f.Evaler, "m").(BindingMap)
+	_, ok := getGlobal(f.Evaler, "m").(bindingsMap)
 	if !ok {
 		t.Errorf("edit:binding-table did not create BindingMap variable")
 	}
 }
 
-func TestCloseListing(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+func TestCloseMode(t *testing.T) {
+	f := setup(t)
 
-	f.Editor.app.MutateState(func(s *cli.State) { s.Addon = cli.Empty{} })
-	evals(f.Evaler, `edit:close-listing`)
+	f.Editor.app.SetAddon(tk.Empty{}, false)
+	evals(f.Evaler, `edit:close-mode`)
 
 	if listing := f.Editor.app.CopyState().Addon; listing != nil {
 		t.Errorf("got listing %v, want nil", listing)
@@ -36,8 +35,7 @@ func TestCloseListing(t *testing.T) {
 }
 
 func TestDumpBuf(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
 	feedInput(f.TTYCtrl, "echo")
 	// Wait until the buffer we want has shown up.
@@ -53,8 +51,7 @@ func TestDumpBuf(t *testing.T) {
 }
 
 func TestInsertRaw(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
 	f.TTYCtrl.Inject(term.K('V', ui.Ctrl))
 	wantBuf := f.MakeBuffer(
@@ -83,16 +80,14 @@ func TestInsertRaw(t *testing.T) {
 }
 
 func TestEndOfHistory(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
 	evals(f.Evaler, `edit:end-of-history`)
 	f.TestTTYNotes(t, "End of history")
 }
 
 func TestKey(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
 	evals(f.Evaler, `k = (edit:key a)`)
 	wantK := ui.K('a')
@@ -102,8 +97,7 @@ func TestKey(t *testing.T) {
 }
 
 func TestRedraw(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
 	evals(f.Evaler,
 		`edit:current-command = echo`,
@@ -118,11 +112,22 @@ func TestRedraw(t *testing.T) {
 		"   vvvv", term.DotHere)
 }
 
-func TestReturnCode(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+func TestClear(t *testing.T) {
+	f := setup(t)
 
-	f.Editor.app.CodeArea().MutateState(func(s *cli.CodeAreaState) {
+	evals(f.Evaler, `edit:current-command = echo`, `edit:clear`)
+	f.TestTTY(t,
+		"~> echo", Styles,
+		"   vvvv", term.DotHere)
+	if cleared := f.TTYCtrl.ScreenCleared(); cleared != 1 {
+		t.Errorf("screen cleared %v times, want 1", cleared)
+	}
+}
+
+func TestReturnCode(t *testing.T) {
+	f := setup(t)
+
+	f.Editor.app.CodeArea().MutateState(func(s *tk.CodeAreaState) {
 		s.Buffer.Content = "test code"
 	})
 	evals(f.Evaler, `edit:return-line`)
@@ -136,8 +141,7 @@ func TestReturnCode(t *testing.T) {
 }
 
 func TestReturnEOF(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
 	evals(f.Evaler, `edit:return-eof`)
 	if _, err := f.Wait(); err != io.EOF {
@@ -146,22 +150,20 @@ func TestReturnEOF(t *testing.T) {
 }
 
 func TestSmartEnter_InsertsNewlineWhenIncomplete(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
-	cli.SetCodeBuffer(f.Editor.app, cli.CodeBuffer{Content: "put [", Dot: 5})
+	f.SetCodeBuffer(tk.CodeBuffer{Content: "put [", Dot: 5})
 	evals(f.Evaler, `edit:smart-enter`)
-	wantBuf := cli.CodeBuffer{Content: "put [\n", Dot: 6}
-	if buf := cli.GetCodeBuffer(f.Editor.app); buf != wantBuf {
+	wantBuf := tk.CodeBuffer{Content: "put [\n", Dot: 6}
+	if buf := f.Editor.app.CodeArea().CopyState().Buffer; buf != wantBuf {
 		t.Errorf("got code buffer %v, want %v", buf, wantBuf)
 	}
 }
 
 func TestSmartEnter_AcceptsCodeWhenWholeBufferIsComplete(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	f := setup(t)
 
-	cli.SetCodeBuffer(f.Editor.app, cli.CodeBuffer{Content: "put []", Dot: 5})
+	f.SetCodeBuffer(tk.CodeBuffer{Content: "put []", Dot: 5})
 	evals(f.Evaler, `edit:smart-enter`)
 	wantCode := "put []"
 	if code, _ := f.Wait(); code != wantCode {
@@ -170,51 +172,50 @@ func TestSmartEnter_AcceptsCodeWhenWholeBufferIsComplete(t *testing.T) {
 }
 
 func TestWordify(t *testing.T) {
-	f := setup()
-	defer f.Cleanup()
+	TestWithSetup(t, setupWordify,
+		That("wordify 'ls str [list]'").Puts("ls", "str", "[list]"),
+		That("wordify foo >&-").Throws(AnyError),
+	)
+}
 
-	evals(f.Evaler, `@words = (edit:wordify 'ls str [list]')`)
-	wantWords := vals.MakeList("ls", "str", "[list]")
-	if words, _ := f.Evaler.Global().Index("words"); !vals.Equal(words, wantWords) {
-		t.Errorf("$words is %v, want %v", words, wantWords)
-	}
+func setupWordify(ev *eval.Evaler) {
+	ev.AddBuiltin(eval.NsBuilder{}.AddGoFn("", "wordify", wordify).Ns())
 }
 
 var bufferBuiltinsTests = []struct {
 	name      string
-	bufBefore cli.CodeBuffer
-	bufAfter  cli.CodeBuffer
+	bufBefore tk.CodeBuffer
+	bufAfter  tk.CodeBuffer
 }{
 	{
 		"move-dot-left",
-		cli.CodeBuffer{Content: "ab", Dot: 1},
-		cli.CodeBuffer{Content: "ab", Dot: 0},
+		tk.CodeBuffer{Content: "ab", Dot: 1},
+		tk.CodeBuffer{Content: "ab", Dot: 0},
 	},
 	{
 		"move-dot-right",
-		cli.CodeBuffer{Content: "ab", Dot: 1},
-		cli.CodeBuffer{Content: "ab", Dot: 2},
+		tk.CodeBuffer{Content: "ab", Dot: 1},
+		tk.CodeBuffer{Content: "ab", Dot: 2},
 	},
 	{
 		"kill-rune-left",
-		cli.CodeBuffer{Content: "ab", Dot: 1},
-		cli.CodeBuffer{Content: "b", Dot: 0},
+		tk.CodeBuffer{Content: "ab", Dot: 1},
+		tk.CodeBuffer{Content: "b", Dot: 0},
 	},
 	{
 		"kill-rune-right",
-		cli.CodeBuffer{Content: "ab", Dot: 1},
-		cli.CodeBuffer{Content: "a", Dot: 1},
+		tk.CodeBuffer{Content: "ab", Dot: 1},
+		tk.CodeBuffer{Content: "a", Dot: 1},
 	},
 }
 
 func TestBufferBuiltins(t *testing.T) {
-	f := setup()
+	f := setup(t)
 	app := f.Editor.app
-	defer f.Cleanup()
 
 	for _, test := range bufferBuiltinsTests {
 		t.Run(test.name, func(t *testing.T) {
-			app.CodeArea().MutateState(func(s *cli.CodeAreaState) {
+			app.CodeArea().MutateState(func(s *tk.CodeAreaState) {
 				s.Buffer = test.bufBefore
 			})
 			evals(f.Evaler, "edit:"+test.name)
