@@ -4,16 +4,14 @@ import (
 	"os"
 	"strings"
 
-	"github.com/elves/elvish/pkg/eval"
-	"github.com/elves/elvish/pkg/fsutil"
+	"src.elv.sh/pkg/eval"
+	"src.elv.sh/pkg/fsutil"
 )
 
 // Calls the passed function for each variable name in namespace ns that can be
 // found from the top context.
 func eachVariableInTop(builtin, global *eval.Ns, ns string, f func(s string)) {
 	switch ns {
-	case "builtin:":
-		builtin.IterateNames(f)
 	case "", ":":
 		global.IterateNames(f)
 		builtin.IterateNames(f)
@@ -48,7 +46,6 @@ func eachVariableInTop(builtin, global *eval.Ns, ns string, f func(s string)) {
 // Calls the passed function for each namespace that can be used from the top
 // context.
 func eachNsInTop(builtin, global *eval.Ns, f func(s string)) {
-	f("builtin:")
 	f("e:")
 	f("E:")
 
